<?php

/*
 * (C) 2013-2015 iDorMobile, telpho GmbH
 *
 * Version: 1.6.0
 *
 */

function GetNetInfo($ipaddress, $netmask, &$netinfo) {

	$ret_value = false;
	$my_net_info = $ipaddress." ".$netmask;

	if (! ereg('^([0-9]{1,3}\.){3}[0-9]{1,3}(( ([0-9]{1,3}\.){3}[0-9]{1,3})|(/[0-9]{1,2}))$',$my_net_info)){
		return false;
	}

	if (ereg("/",$my_net_info)){  //if cidr type mask
		$dq_host = strtok("$my_net_info", "/");
		$cdr_nmask = strtok("/");
		if (!($cdr_nmask >= 0 && $cdr_nmask <= 32)){
			return false;
		}
		$bin_nmask=cdrtobin($cdr_nmask);
		$bin_wmask=binnmtowm($bin_nmask);
	} else { //Dotted quad mask?
		$dqs=explode(" ", $my_net_info);
		$dq_host=$dqs[0];
		$bin_nmask=dqtobin($dqs[1]);
		$bin_wmask=binnmtowm($bin_nmask);
		if (ereg("0",rtrim($bin_nmask, "0"))) {  //Wildcard mask then? hmm?
			$bin_wmask=dqtobin($dqs[1]);
			$bin_nmask=binwmtonm($bin_wmask);
			if (ereg("0",rtrim($bin_nmask, "0"))){ //If it's not wcard, whussup?
				return false;
			}
		}
		$cdr_nmask=bintocdr($bin_nmask);
	}

	//Check for valid $dq_host
	if(! ereg('^0.',$dq_host)){
		foreach( explode(".",$dq_host) as $octet ){
			if($octet > 255){ 
				return false;
			}
		
		}
	}

	$bin_host=dqtobin($dq_host);
	$bin_bcast=(str_pad(substr($bin_host,0,$cdr_nmask),32,1));
	$bin_net=(str_pad(substr($bin_host,0,$cdr_nmask),32,0));
	$bin_first=(str_pad(substr($bin_net,0,31),32,1));
	$bin_last=(str_pad(substr($bin_bcast,0,31),32,0));
	$host_total=(bindec(str_pad("",(32-$cdr_nmask),1)) - 1);

	if ($host_total <= 0){  //Takes care of 31 and 32 bit masks.
		$bin_first="N/A" ; $bin_last="N/A" ; $host_total="N/A";
		if ($bin_net === $bin_bcast) $bin_bcast="N/A";
	}

	//Determine Class
	if (ereg('^0',$bin_net)){
		$class="A";
		$dotbin_net = substr(dotbin($bin_net,$cdr_nmask),1) ;
	}elseif (ereg('^10',$bin_net)){
		$class="B";
		$dotbin_net = substr(dotbin($bin_net,$cdr_nmask),2) ;
	}elseif (ereg('^110',$bin_net)){
		$class="C";
		$dotbin_net = substr(dotbin($bin_net,$cdr_nmask),3) ;
	}elseif (ereg('^1110',$bin_net)){
		$class ="D";
		$dotbin_net = substr(dotbin($bin_net,$cdr_nmask),4) ;
	}else{
		$class="E";
		$dotbin_net = substr(dotbin($bin_net,$cdr_nmask),4) ;
	}

	$netinfo['ipaddress'] = $dq_host;
	$netinfo['network'] = bintodq($bin_net);
	$netinfo['broadcast'] = bintodq($bin_bcast);
	
	/*
	// Print Results
	print "Adresse:   ".$dq_host." ".dotbin($bin_host,$cdr_nmask)."\r\n";

	print "Netmask:   ".$bin_nmask." ".dotbin($bin_nmask, $cdr_nmask)."\r\n";

	print "Wildcard:  ".bintodq($bin_wmask)." ".dotbin($bin_wmask, $cdr_nmask)."\r\n";

	print "Network:   ".bintodq($bin_net)." ".$dotbin_net."\r\n";

	print "Broadcast: ".bintodq($bin_bcast)." ".dotbin($bin_bcast, $cdr_nmask)."\r\n";

	print "HostMin:   ".bintodq($bin_first)." ".dotbin($bin_first, $cdr_nmask)."\r\n";
	print "HostMax:   ".bintodq($bin_last)." ".dotbin($bin_last, $cdr_nmask)."\r\n";
	print "Hosts/Net: ".$host_total."\r\n";

	print "$end";
	*/
	
	return true;
}

function binnmtowm($binin){
	$binin=rtrim($binin, "0");
	if (!ereg("0",$binin) ){
		return str_pad(str_replace("1","0",$binin), 32, "1");
	} else return "1010101010101010101010101010101010101010";
}

function bintocdr ($binin){
	return strlen(rtrim($binin,"0"));
}

function bintodq ($binin) {
	if ($binin=="N/A") return $binin;
	$binin=explode(".", chunk_split($binin,8,"."));
	for ($i=0; $i<4 ; $i++) {
		$dq[$i]=bindec($binin[$i]);
	}
        return implode(".",$dq) ;
}

function bintoint ($binin){
        return bindec($binin);
}

function binwmtonm($binin){
	$binin=rtrim($binin, "1");
	if (!ereg("1",$binin)){
		return str_pad(str_replace("0","1",$binin), 32, "0");
	} else return "1010101010101010101010101010101010101010";
}

function cdrtobin ($cdrin){
	return str_pad(str_pad("", $cdrin, "1"), 32, "0");
}

function dotbin($binin,$cdr_nmask){
	// splits 32 bit bin into dotted bin octets
	if ($binin=="N/A") return $binin;
	$oct=rtrim(chunk_split($binin,8,"."),".");
	if ($cdr_nmask > 0){
		$offset=sprintf("%u",$cdr_nmask/8) + $cdr_nmask ;
		return substr($oct,0,$offset ) . "&nbsp;&nbsp;&nbsp;" . substr($oct,$offset) ;
	} else {
	return $oct;
	}
}

function dqtobin($dqin) {
        $dq = explode(".",$dqin);
        for ($i=0; $i<4 ; $i++) {
           $bin[$i]=str_pad(decbin($dq[$i]), 8, "0", STR_PAD_LEFT);
        }
        return implode("",$bin);
}

function inttobin ($intin) {
        return str_pad(decbin($intin), 32, "0", STR_PAD_LEFT);
}

?>